; modify mp3 files as per following specification
; input filename format: 157 - artist - song.mp3
; output filename format: 2-11 song.mp3
;   2 = CD number, 11 = track number, all tags updated
Opt("MustDeclareVars", 0)

; Constants
Const $numDelim		= '-'
Const $varArtist	= 'Various Artists'
Const $mp3Ext		= '.mp3'
Const $metamp3		= 'metamp3.exe'
Const $defToolPath	= 'C:\Program Files\REACT2\tools\'

; Variables
Local $tracksPerCd		= 0	; tracks per CD
Local $tracksThisCd		= 0	; tracks on CD processing (differs from $tracksPerCd for last CD in set)
Local $tracksNum		= 0	; total number of tracks
Local $totalCd			= 0	; total number of CDs
Local $currentTrack		= 1 ; current track processing
Local $currentCd		= 1	; current CD processing
Local $tracksDone		= 0	; feedback counter

Local $toolPath			= ''
Local $filePath			= ''
Local $writePath		= ''
Local $fileName			= ''
Local $newFileName		= ''
Local $fileTime			= ''
Local $coverFile		= ''

Local $artist			= ''
Local $title			= ''
Local $trackCountStr	= ''
Local $cdCountStr		= ''
Local $artistStr		= ''
Local $titleStr			= ''
Local $albumArtistStr	= ''
Local $coverStr			= ''

Local $cd				= StringToASCIIArray("00")
Local $fileHandle		= -1
Local $tagCover			= False

; get toolPath from command line
If $CmdLine[0] = 1 Then
	$toolPath = $CmdLine[1] & '\'
Else
	$toolPath = $defToolPath
EndIf

; get filePath and album art
$filePath = FileSelectFolder("Directory which contains the music files", "", 4, "::{450D8FBA-AD25-11D0-98A8-0800361B1103}") & '\'
If @error >= 1 Then Exit
$writePath = FileSelectFolder("Directory to write the modified music files to", "", 4+1, "::{450D8FBA-AD25-11D0-98A8-0800361B1103}") & '\'
If @error >= 1 Then Exit
$coverFile = FileOpenDialog("Album Art file", "::{450D8FBA-AD25-11D0-98A8-0800361B1103}", "Images (*.jpg)")
If @error >= 1 Then $coverFile = ''
$tagCover = $coverFile <> ''

; input cd # and total # of CDs
$tracksPerCd = InputBox("Tracks per CD", "Enter #", "", " M4")
If @error >= 1	Then Exit
$tracksNum = InputBox("Total number of Tracks", "Enter #", "", " M4")
If @error >= 1 Then Exit
$totalCd = Int($tracksNum / $tracksPerCd + 0.9999)
;MsgBox(0,"Info", $tracksNum & ' tracks, ' & $tracksPerCd & ' per CD: ' & $totalCd & ' CDs')

$currentCd = 1
$currentTrack = 1
$fileHandle = FileFindFirstFile($filePath & '*')  

; Loop if the search was successful
If $fileHandle >= 0 Then
	While 1
		$fileName = FileFindNextFile($fileHandle)
		If @error Then ExitLoop

		; check filename and extract artist & song title
		$trArr = StringSplit($fileName, ' - ', 1)
		If $trArr[0] < 3 Then
			MsgBox(49, "Error", "Invalid file name: " & $fileName)
			ExitLoop
		EndIf
		$artist = StringStripWS($trArr[2], 3)
		$title = StringStripWS($trArr[3], 3)
		$title = StringLeft($title, StringLen($title)-StringLen($mp3Ext))
		
		; compose new file name
		$cd[0] = 48 + Int($currentTrack / 10)
		$cd[1] = 48 + Mod($currentTrack, 10)
		$newFileName = $currentCd & $numDelim & StringFromASCIIArray($cd) & ' ' & $title & $mp3Ext
		;MsgBox(48, "Info", 'artist=' & $artist & ', track=' & $fileName & ', file=' & $newFileName)

		; prepare meta tags content
		If $currentCd = $totalCd Then
			$tracksThisCd = $tracksNum - (($totalCd - 1) * $tracksPerCd)
		Else
			$tracksThisCd = $tracksPerCd
		EndIf
		$trackCountStr	= ' --frame "TRCK:' & $currentTrack & '/' & $tracksThisCd & '"'
		$cdCountStr		= ' --frame "TPOS:' & $currentCd & '/' & $totalCd & '"'
		$artistStr		= ' --frame "TPE1:' & $artist & '"'
		$titleStr		= ' --frame "TIT2:' & $title & '"'
		$albumArtistStr	= ' --frame "TPE2:' & $varArtist & '" --frame "TXXX[album artist]:' & $varArtist & '"'
		If $tagCover Then
			$coverStr = ' --frame "APIC[]{3}:' & $coverFile & '"'
		Else
			$coverStr = ''
		EndIf
		

		; fix created and modified time stamps; update meta tags
		$fileTime = FileGetTime($filePath & $fileName, 0, 1)
;	MsgBox(48, "",  $toolPath & $metamp3 & $trackCountStr & $cdCountStr & $artistStr & $titleStr & $albumArtistStr & $coverStr & ' "' & $filePath & $fileName & '"')
			RunWait($toolPath & $metamp3 & $trackCountStr & $cdCountStr & $artistStr & $titleStr & $albumArtistStr & $coverStr & ' "' & $filePath & $fileName & '"')
		If @error Then
			MsgBox(49, 'Error', '"' & $toolPath & $metamp3 & '" not found')
			ExitLoop
		EndIf
		FileSetTime($filePath & $fileName, $fileTime, 0)
		FileSetTime($filePath & $fileName, $fileTime, 1)

		; rename the file
		If Not FileMove($filePath & $fileName, $writePath & $newFileName) Then
			If Not MsgBox(49, 'Warning', 'Failed to move: "' & $filePath & $fileName & '"') = 1 Then Exit EndIf
		EndIf


		; increment cd and track numbers
		If $currentTrack < $tracksPerCd Then
			$currentTrack = $currentTrack + 1
		Else
			$currentTrack = 1
			$currentCd = $currentCd + 1
		EndIf
		$tracksDone = $tracksDone + 1
	WEnd
	; Close the search handle
	FileClose($fileHandle)
EndIf

MsgBox(64, 'Done', $tracksDone & '/' & $tracksNum & ' files in directory "' & $filePath & '" renamed and retagged', 10)
